;/////////////////////////////////////////////////////////
;//                                                     //
;//               Freepascal barebone OS                //
;//                      stub.asm                       //
;//                                                     //
;/////////////////////////////////////////////////////////
;//
;//     By:             De Deyn Kim <kimdedeyn@skynet.be>
;//     License:        Public domain
;//

;
; Kernel stub
;

;
; We are in 32bits protected mode
;
[bits 32]

;
; Export entrypoint
;
[global kstart]
[global flush_gdt]
[global load_idt]

;
; Import kernel entrypoint
;
[extern kernel_main]
[extern gdt_ptr]
[extern idt_ptr]
[extern fault_handler]
;
; Posible multiboot header flags
;
MULTIBOOT_MODULE_ALIGN          equ     1<<0
MULTIBOOT_MEMORY_MAP            equ     1<<1
MULTIBOOT_GRAPHICS_FIELDS       equ     1<<2
MULTIBOOT_ADDRESS_FIELDS        equ     1<<16

;
; Multiboot header defines
;
MULTIBOOT_HEADER_MAGIC          equ     0x1BADB002
MULTIBOOT_HEADER_FLAGS          equ     MULTIBOOT_MODULE_ALIGN | MULTIBOOT_MEMORY_MAP
MULTIBOOT_HEADER_CHECKSUM       equ     -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)

;
; Kernel stack size
;
KERNEL_STACKSIZE                equ     0x4000

section .text

;
; Multiboot header
;
align 4
dd MULTIBOOT_HEADER_MAGIC
dd MULTIBOOT_HEADER_FLAGS
dd MULTIBOOT_HEADER_CHECKSUM

;
; Entrypoint
;
kstart:
    mov esp, KERNEL_STACK+KERNEL_STACKSIZE  ;Create kernel stack
    push eax                                ;Multiboot magic number
    push ebx                                ;Multiboot info
    call kernel_main                               ;Call kernel entrypoint
    cli                                     ;Clear interrupts
    hlt                                     ;Halt machine

flush_gdt:
    lgdt [gdt_ptr]        ; Load the GDT with our '_gp' which is a special pointer
    mov ax, 0x10      ; 0x10 is the offset in the GDT to our data segment
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov ss, ax
    jmp 0x08:flush2   ; 0x08 is the offset to our code segment: Far jump!
flush2:
    ret               ; Returns back to the Pascal code!

load_idt:
    lidt [idt_ptr]
    ret

global isr0
global isr1
global isr2
global isr3
global isr4
global isr5
global isr6
global isr7
global isr8
global isr9
global isr10
global isr11
global isr12
global isr13
global isr14
global isr15
global isr16
global isr17
global isr18
global isr19
global isr20
global isr21
global isr22
global isr23
global isr24
global isr25
global isr26
global isr27
global isr28
global isr29
global isr30
global isr31

;  0: Divide By Zero Exception
isr0:
    cli
    push byte 0
    push byte 0
    jmp isr_common_stub

;  1: Debug Exception
isr1:
    cli
    push byte 1
    push byte 0
    jmp isr_common_stub

;  2: Non Maskable Interrupt Exception
isr2:
    cli
    push byte 2
    push byte 0
    jmp isr_common_stub

;  3: Int 3 Exception
isr3:
    cli
    push byte 3
    push byte 0
    jmp isr_common_stub

;  4: INTO Exception
isr4:
    cli
    push byte 4
    push byte 0
    jmp isr_common_stub

;  5: Out of Bounds Exception
isr5:
    cli
    push byte 5
    push byte 0
    jmp isr_common_stub

;  6: Invalid Opcode Exception
isr6:
    cli
    push byte 6
    push byte 0
    jmp isr_common_stub

;  7: Coprocessor Not Available Exception
isr7:
    cli
    push byte 7
    push byte 0
    jmp isr_common_stub

;  8: Double Fault Exception (With Error Code!)
isr8:
    cli
    push byte 8
    jmp isr_common_stub

;  9: Coprocessor Segment Overrun Exception
isr9:
    cli
    push byte 9
    push byte 0
    jmp isr_common_stub

; 10: Bad TSS Exception (With Error Code!)
isr10:
    cli
    push byte 10
    jmp isr_common_stub

; 11: Segment Not Present Exception (With Error Code!)
isr11:
    cli
    push byte 11
    jmp isr_common_stub

; 12: Stack Fault Exception (With Error Code!)
isr12:
    cli
    push byte 12
    jmp isr_common_stub

; 13: General Protection Fault Exception (With Error Code!)
isr13:
    cli
    push byte 13
    jmp isr_common_stub

; 14: Page Fault Exception (With Error Code!)
isr14:
    cli
    push byte 14
    jmp isr_common_stub

; 15: Reserved Exception
isr15:
    cli
    push byte 15
    push byte 0
    jmp isr_common_stub

; 16: Floating Point Exception
isr16:
    cli
    push byte 16
    push byte 0
    jmp isr_common_stub

; 17: Alignment Check Exception
isr17:
    cli
    push byte 17
    push byte 0
    jmp isr_common_stub

; 18: Machine Check Exception
isr18:
    cli
    push byte 18
    push byte 0
    jmp isr_common_stub

; 19: Reserved
isr19:
    cli
    push byte 19
    push byte 0
    jmp isr_common_stub

; 20: Reserved
isr20:
    cli
    push byte 20
    push byte 0
    jmp isr_common_stub

; 21: Reserved
isr21:
    cli
    push byte 21
    push byte 0
    jmp isr_common_stub

; 22: Reserved
isr22:
    cli
    push byte 22
    push byte 0
    jmp isr_common_stub

; 23: Reserved
isr23:
    cli
    push byte 23
    push byte 0
    jmp isr_common_stub

; 24: Reserved
isr24:
    cli
    push byte 24
    push byte 0
    jmp isr_common_stub

; 25: Reserved
isr25:
    cli
    push byte 25
    push byte 0
    jmp isr_common_stub

; 26: Reserved
isr26:
    cli
    push byte 26
    push byte 0
    jmp isr_common_stub

; 27: Reserved
isr27:
    cli
    push byte 27
    push byte 0
    jmp isr_common_stub

; 28: Reserved
isr28:
    cli
    push byte 28
    push byte 0
    jmp isr_common_stub

; 29: Reserved
isr29:
    cli
    push byte 29
    push byte 0
    jmp isr_common_stub

; 30: Reserved
isr30:
    cli
    push byte 30
    push byte 0
    jmp isr_common_stub

; 31: Reserved
isr31:
    cli
    push byte 31
    push byte 0
    jmp isr_common_stub

isr_common_stub:
    pusha

    mov eax, esp
    push eax
    push gs
    push fs
    push es
    push ds

    mov ax, 0x10
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax

	mov eax,fault_handler
    call eax

    pop ds
    pop es
    pop fs
    pop gs
    pop eax

    popa
    add esp, 8
    iret

section .bss

;
; Kernel stack location
;
align 32
KERNEL_STACK:
    resb KERNEL_STACKSIZE
